
def parse_slice(selector):
    l = selector.split(':')

    def f(x):
        return None if x == '' else int(x)
    return slice(f(l[0]), f(l[1]))


indice = [(slice(0, 20, None), slice(20, 36, None)),
          (slice(36, 56, None), slice(56, 72, None)),
          (slice(72, 92, None), slice(92, 108, None)),
          (slice(108, 128, None), slice(128, 144, None)),
          (slice(144, 164, None), slice(164, 180, None))]




import rasterio
import riomucho
import numpy as np
import pymannkendall as mk
import scipy
from scipy import stats
from .utils import parse_slice

np.seterr(all='raise')
scipy.seterr(all='raise')


def basic_run(datum, window, ij, g_args):
    print(window)
    data = datum[0]
    out = np.zeros((2, *data.shape[1:]))
    out.fill(g_args['nodata'])

    sliceObj = parse_slice(g_args['selector'])
    sliceObj2 = parse_slice(g_args['selector2'])

    for i, j in np.ndindex(data.shape[1:]):
        pixel = data[:, i, j]
        a = pixel[sliceObj]
        b = pixel[sliceObj2]
        try:
            r, p = stats.pearsonr(a, b)
        except:
            pass
        else:
            out[0, i, j] = r
            out[1, i, j] = p

    return out.astype(np.float32)


def run_pearson(in_file, out_file, selector, selector2):
    float32_min = np.finfo(np.float32).min
    # get windows from an input
    with rasterio.open(in_file) as src:
        # grabbing the windows as an example. Default behavior is identical.
        windows = [[window, ij] for ij, window in src.block_windows()]
        options = src.meta
        # since we are only writing to 2 bands
        options.update(count=2, dtype=rasterio.float32, nodata=float32_min)

    global_args = {
        'selector': selector,
        'selector2': selector2,
        'nodata': float32_min
    }

    processes = 4

    # run it
    with riomucho.RioMucho([in_file], out_file, basic_run,
                           windows=windows,
                           global_args=global_args,
                           options=options) as rm:

        rm.run(processes)



import numpy as np


def trend_remap(slope, p) -> int:
    if np.isnan(slope) or np.isnan(p):
        return np.nan
    if p <= 0.01 and slope <= 0:
        return 1
    elif 0.01 < p and p <= 0.05 and slope <= 0:
        return 2
    elif 0.05 < p and slope <= 0:
        return 3
    elif p <= 0.01 and slope > 0:
        return 6
    elif 0.01 < p and p <= 0.05 and slope > 0:
        return 5
    elif 0.05 < p and slope > 0:
        return 4
    else:
        return 0


def pearsonr_remap(r, p) -> int:
    if p < 0.01:
        out = 1 if r < 0 else 6
    elif 0.01 <= p < 0.05:
        out = 2 if r < 0 else 5
    else:
        out = 3 if r < 0 else 4
    return out


def mk_result_remap(trend) -> int:
    data = {
        'increasing': 3,
        'decreasing': 1,
        'no trend': 2,
    }
    return data.get(trend, 0)
